=== modified file 'NEWS'
--- NEWS	2007-03-27 21:45:59 +0000
+++ NEWS	2007-03-28 03:12:39 +0000
@@ -5,6 +5,10 @@
     * bzrlib API compatability with 0.8 has been dropped, cleaning up some
       code paths. (Robert Collins)
 
+    * Rewritten chroot transport that prevents accidental chroot escapes when
+      using urlutils against the transport's url.
+      (Robert Collins, Andrew Bennetts)
+
     * ``LockableFiles.lock_write()`` now accepts a ``token`` keyword argument,
       so that a seperate LockableFiles instance can share a lock if it has the
       right token.  (Andrew Bennetts, Robert Collins)

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2007-03-27 09:46:24 +0000
+++ bzrlib/tests/test_transport.py	2007-03-28 03:12:39 +0000
@@ -22,16 +22,15 @@
 
 import bzrlib
 from bzrlib import urlutils
-from bzrlib.errors import (
-    ConnectionError,
-    DependencyNotPresent,
-    FileExists,
-    InvalidURLJoin,
-    NoSuchFile,
-    PathNotChild,
-    TransportNotPossible,
-    UnsupportedProtocol,
-    )
+from bzrlib.errors import (ConnectionError,
+                           DependencyNotPresent,
+                           FileExists,
+                           InvalidURLJoin,
+                           NoSuchFile,
+                           PathNotChild,
+                           TransportNotPossible,
+                           UnsupportedProtocol,
+                           )
 from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.transport import (_CoalescedOffset,
                               _get_protocol_handlers,
@@ -72,7 +71,7 @@
             _set_protocol_handlers(my_handlers)
             register_lazy_transport('foo', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
             register_lazy_transport('bar', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
-            self.assertEqual([SampleHandler.__module__],
+            self.assertEqual([SampleHandler.__module__, 'bzrlib.transport.chroot'],
                              _get_transport_modules())
         finally:
             _set_protocol_handlers(handlers)
@@ -379,21 +378,6 @@
         server.tearDown()
 
 
-class FakeTransport(object):
-    # XXX: FakeTransport copied from test_wsgi.py
-
-    def __init__(self):
-        self.calls = []
-        self.base = 'fake:///'
-
-    def abspath(self, relpath):
-        return 'fake:///' + relpath
-
-    def clone(self, relpath):
-        self.calls.append(('clone', relpath))
-        return self
-
-
 class ReadonlyDecoratorTransportTest(TestCase):
     """Readonly decoration specific tests."""
 

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2007-03-27 09:46:24 +0000
+++ bzrlib/tests/test_transport_implementations.py	2007-03-28 03:12:39 +0000
@@ -38,7 +38,7 @@
 from bzrlib.symbol_versioning import zero_eleven
 from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.tests.test_transport import TestTransportImplementation
-from bzrlib.transport import memory, chroot
+from bzrlib.transport import memory
 from bzrlib.smart import medium
 import bzrlib.transport
 

=== modified file 'bzrlib/tests/test_wsgi.py'
--- bzrlib/tests/test_wsgi.py	2007-03-27 09:46:24 +0000
+++ bzrlib/tests/test_wsgi.py	2007-03-28 03:12:39 +0000
@@ -85,8 +85,8 @@
     def test_smart_wsgi_app_uses_given_relpath(self):
         # XXX XXX XXX update comment
         # The SmartWSGIApp should use the "bzrlib.relpath" field from the
-        # WSGI environ to construct the transport for this request, by cloning
-        # its base transport with the given relpath.
+        # WSGI environ to clone from its backing transport to get a specific
+        # transport for this request.
         transport = FakeTransport()
         wsgi_app = wsgi.SmartWSGIApp(transport)
         wsgi_app.backing_transport = transport

=== modified file 'bzrlib/transport/http/wsgi.py'
--- bzrlib/transport/http/wsgi.py	2007-03-27 09:46:24 +0000
+++ bzrlib/transport/http/wsgi.py	2007-03-28 03:12:39 +0000
@@ -98,12 +98,13 @@
         self.chroot_server = chroot.ChrootServer(backing_transport)
         self.chroot_server.setUp()
         self.backing_transport = get_transport(self.chroot_server.get_url())
-        # XXX: explain why not:
+        # While the chroot server can technically be torn down at this point,
+        # as all it does is remove the scheme registration from transport's 
+        # protocol dictionary, we dont *just in case* there are parts of 
+        # bzrlib that will invoke 'get_transport' on urls rather than cloning
+        # around the existing transport.
         #self.chroot_server.tearDown()
 
-        #self.backing_transport = chroot.ChrootTransportDecorator(
-        #    'chroot+' + backing_transport.base, _decorated=backing_transport)
-
     def __call__(self, environ, start_response):
         """WSGI application callable."""
         if environ['REQUEST_METHOD'] != 'POST':

